# State-of-the-art for datasets < 100k samples. Requires a GPU with at least 20 GB VRAM.
hyperparameter_portfolio_zeroshot_gpu_2025_12_18 = {
    "TABDPT": [
        {
            "ag_args": {"name_suffix": "_c1", "priority": -3},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": False},
        },
        {
            "ag_args": {"name_suffix": "_r20", "priority": -5},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": False},
            "clip_sigma": 8,
            "feature_reduction": "subsample",
            "missing_indicators": False,
            "normalizer": "quantile-uniform",
            "permute_classes": False,
            "temperature": 0.5,
        },
        {
            "ag_args": {"name_suffix": "_r1", "priority": -7},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": False},
            "clip_sigma": 16,
            "feature_reduction": "subsample",
            "missing_indicators": False,
            "normalizer": "log1p",
            "permute_classes": False,
            "temperature": 0.5,
        },
        {
            "ag_args": {"name_suffix": "_r15", "priority": -9},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": False},
            "clip_sigma": 16,
            "feature_reduction": "subsample",
            "missing_indicators": False,
            "normalizer": "standard",
            "permute_classes": True,
            "temperature": 0.7,
        },
        {
            "ag_args": {"name_suffix": "_r22", "priority": -11},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": False},
            "clip_sigma": 8,
            "feature_reduction": "pca",
            "missing_indicators": True,
            "normalizer": "robust",
            "permute_classes": False,
            "temperature": 0.5,
        },
    ],
    "TABICL": [{"ag_args": {"name_suffix": "_c1", "priority": -4}, "ag_args_ensemble": {"refit_folds": True}}],
    "MITRA": [
        {
            "ag_args": {"name_suffix": "_c1", "priority": -12},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": True},
        }
    ],
    "TABM": [
        {
            "ag_args": {"name_suffix": "_r99", "priority": -13},
            "amp": False,
            "arch_type": "tabm-mini",
            "batch_size": "auto",
            "d_block": 880,
            "d_embedding": 24,
            "dropout": 0.10792355695428629,
            "gradient_clipping_norm": 1.0,
            "lr": 0.0013641856391615784,
            "n_blocks": 5,
            "num_emb_n_bins": 16,
            "num_emb_type": "pwl",
            "patience": 16,
            "share_training_batches": False,
            "tabm_k": 32,
            "weight_decay": 0.0,
        },
        {
            "ag_args": {"name_suffix": "_r124", "priority": -17},
            "amp": False,
            "arch_type": "tabm-mini",
            "batch_size": "auto",
            "d_block": 208,
            "d_embedding": 16,
            "dropout": 0.0,
            "gradient_clipping_norm": 1.0,
            "lr": 0.00042152744054701374,
            "n_blocks": 2,
            "num_emb_n_bins": 109,
            "num_emb_type": "pwl",
            "patience": 16,
            "share_training_batches": False,
            "tabm_k": 32,
            "weight_decay": 0.00014007839435474664,
        },
        {
            "ag_args": {"name_suffix": "_r69", "priority": -21},
            "amp": False,
            "arch_type": "tabm-mini",
            "batch_size": "auto",
            "d_block": 848,
            "d_embedding": 28,
            "dropout": 0.40215621636031007,
            "gradient_clipping_norm": 1.0,
            "lr": 0.0010413640454559532,
            "n_blocks": 3,
            "num_emb_n_bins": 18,
            "num_emb_type": "pwl",
            "patience": 16,
            "share_training_batches": False,
            "tabm_k": 32,
            "weight_decay": 0.0,
        },
        {
            "ag_args": {"name_suffix": "_r184", "priority": -24},
            "amp": False,
            "arch_type": "tabm-mini",
            "batch_size": "auto",
            "d_block": 864,
            "d_embedding": 24,
            "dropout": 0.0,
            "gradient_clipping_norm": 1.0,
            "lr": 0.0019256819924656217,
            "n_blocks": 3,
            "num_emb_n_bins": 3,
            "num_emb_type": "pwl",
            "patience": 16,
            "share_training_batches": False,
            "tabm_k": 32,
            "weight_decay": 0.0,
        },
        {
            "ag_args": {"name_suffix": "_r34", "priority": -26},
            "amp": False,
            "arch_type": "tabm-mini",
            "batch_size": "auto",
            "d_block": 896,
            "d_embedding": 8,
            "dropout": 0.0,
            "gradient_clipping_norm": 1.0,
            "lr": 0.002459175026451607,
            "n_blocks": 4,
            "num_emb_n_bins": 104,
            "num_emb_type": "pwl",
            "patience": 16,
            "share_training_batches": False,
            "tabm_k": 32,
            "weight_decay": 0.0006299584388562901,
        },
    ],
    "GBM_PREP": [
        {
            "ag.prep_params": [
                [
                    [["ArithmeticFeatureGenerator", {}]],
                    [
                        ["CategoricalInteractionFeatureGenerator", {"passthrough": True}],
                        ["OOFTargetEncodingFeatureGenerator", {}],
                    ],
                ]
            ],
            "ag.prep_params.passthrough_types": {"invalid_raw_types": ["category", "object"]},
            "ag_args": {"name_suffix": "_r13", "priority": -14},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": True},
            "bagging_fraction": 0.9923026236907,
            "bagging_freq": 1,
            "cat_l2": 0.014290368488,
            "cat_smooth": 1.8662939903973,
            "extra_trees": True,
            "feature_fraction": 0.5533919718605,
            "lambda_l1": 0.914411672958,
            "lambda_l2": 1.90439560009,
            "learning_rate": 0.0193225778401,
            "max_cat_to_onehot": 18,
            "min_data_in_leaf": 28,
            "min_data_per_group": 54,
            "num_leaves": 64,
        },
        {
            "ag.prep_params": [
                [
                    [["ArithmeticFeatureGenerator", {}]],
                    [
                        ["CategoricalInteractionFeatureGenerator", {"passthrough": True}],
                        ["OOFTargetEncodingFeatureGenerator", {}],
                    ],
                ]
            ],
            "ag.prep_params.passthrough_types": {"invalid_raw_types": ["category", "object"]},
            "ag_args": {"name_suffix": "_r41", "priority": -16},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": True},
            "bagging_fraction": 0.7215411996558,
            "bagging_freq": 1,
            "cat_l2": 1.887369154362,
            "cat_smooth": 0.0278693980873,
            "extra_trees": True,
            "feature_fraction": 0.4247583287144,
            "lambda_l1": 0.1129800247772,
            "lambda_l2": 0.2623265718536,
            "learning_rate": 0.0074201920651,
            "max_cat_to_onehot": 9,
            "min_data_in_leaf": 15,
            "min_data_per_group": 10,
            "num_leaves": 8,
        },
        {
            "ag.prep_params": [
                [
                    [["ArithmeticFeatureGenerator", {}]],
                    [
                        ["CategoricalInteractionFeatureGenerator", {"passthrough": True}],
                        ["OOFTargetEncodingFeatureGenerator", {}],
                    ],
                ]
            ],
            "ag.prep_params.passthrough_types": {"invalid_raw_types": ["category", "object"]},
            "ag_args": {"name_suffix": "_r31", "priority": -18},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": True},
            "bagging_fraction": 0.9591526242875,
            "bagging_freq": 1,
            "cat_l2": 1.8962346412823,
            "cat_smooth": 0.0215219089995,
            "extra_trees": False,
            "feature_fraction": 0.5791844062459,
            "lambda_l1": 0.938461750637,
            "lambda_l2": 0.9899852075056,
            "learning_rate": 0.0397613094741,
            "max_cat_to_onehot": 27,
            "min_data_in_leaf": 1,
            "min_data_per_group": 39,
            "num_leaves": 16,
        },
        {
            "ag.prep_params": [],
            "ag_args": {"name_suffix": "_r21", "priority": -20},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": True},
            "bagging_fraction": 0.7111549514262,
            "bagging_freq": 1,
            "cat_l2": 0.8679131150136,
            "cat_smooth": 48.7244965504817,
            "extra_trees": False,
            "feature_fraction": 0.425140839263,
            "lambda_l1": 0.5140528525242,
            "lambda_l2": 0.5134051978198,
            "learning_rate": 0.0134375321277,
            "max_cat_to_onehot": 16,
            "min_data_in_leaf": 2,
            "min_data_per_group": 32,
            "num_leaves": 20,
        },
        {
            "ag.prep_params": [
                [
                    [["ArithmeticFeatureGenerator", {}]],
                    [
                        ["CategoricalInteractionFeatureGenerator", {"passthrough": True}],
                        ["OOFTargetEncodingFeatureGenerator", {}],
                    ],
                ]
            ],
            "ag.prep_params.passthrough_types": {"invalid_raw_types": ["category", "object"]},
            "ag_args": {"name_suffix": "_r17", "priority": -23},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": True},
            "bagging_fraction": 0.9277474245702,
            "bagging_freq": 1,
            "cat_l2": 0.0731876168104,
            "cat_smooth": 0.1369210915339,
            "extra_trees": False,
            "feature_fraction": 0.6680440910385,
            "lambda_l1": 0.0125057410295,
            "lambda_l2": 0.7157181359874,
            "learning_rate": 0.0351342879995,
            "max_cat_to_onehot": 20,
            "min_data_in_leaf": 1,
            "min_data_per_group": 2,
            "num_leaves": 64,
        },
    ],
    "CAT": [{"ag_args": {"name_suffix": "_c1", "priority": -15}}],
    "GBM": [
        {
            "ag_args": {"name_suffix": "_r73", "priority": -19},
            "bagging_fraction": 0.7295548973583,
            "bagging_freq": 1,
            "cat_l2": 1.8025485263237,
            "cat_smooth": 59.6178463268351,
            "extra_trees": False,
            "feature_fraction": 0.8242607305914,
            "lambda_l1": 0.7265522905459,
            "lambda_l2": 0.3492160682092,
            "learning_rate": 0.0068803786367,
            "max_cat_to_onehot": 16,
            "min_data_in_leaf": 1,
            "min_data_per_group": 10,
            "num_leaves": 24,
        },
        {
            "ag_args": {"name_suffix": "_r37", "priority": -22},
            "bagging_fraction": 0.8096374561947,
            "bagging_freq": 1,
            "cat_l2": 1.6385754694703,
            "cat_smooth": 16.1922506671724,
            "extra_trees": True,
            "feature_fraction": 0.885927003286,
            "lambda_l1": 0.0430386950502,
            "lambda_l2": 0.2507506811761,
            "learning_rate": 0.0079622660542,
            "max_cat_to_onehot": 23,
            "min_data_in_leaf": 7,
            "min_data_per_group": 49,
            "num_leaves": 6,
        },
        {
            "ag_args": {"name_suffix": "_r162", "priority": -25},
            "bagging_fraction": 0.7552878818396,
            "bagging_freq": 1,
            "cat_l2": 0.0081083103544,
            "cat_smooth": 75.7373446363438,
            "extra_trees": False,
            "feature_fraction": 0.6171258454584,
            "lambda_l1": 0.1071522383181,
            "lambda_l2": 1.7882554584069,
            "learning_rate": 0.0229328987255,
            "max_cat_to_onehot": 24,
            "min_data_in_leaf": 23,
            "min_data_per_group": 2,
            "num_leaves": 125,
        },
        {
            "ag_args": {"name_suffix": "_r57", "priority": -27},
            "bagging_fraction": 0.8515739264605,
            "bagging_freq": 1,
            "cat_l2": 0.2263901847144,
            "cat_smooth": 1.7397457971767,
            "extra_trees": True,
            "feature_fraction": 0.6284015946887,
            "lambda_l1": 0.6935431676756,
            "lambda_l2": 1.7605230133162,
            "learning_rate": 0.0294830579218,
            "max_cat_to_onehot": 52,
            "min_data_in_leaf": 8,
            "min_data_per_group": 3,
            "num_leaves": 43,
        },
        {
            "ag_args": {"name_suffix": "_r33", "priority": -28},
            "bagging_fraction": 0.9625293420216,
            "bagging_freq": 1,
            "cat_l2": 0.1236875455555,
            "cat_smooth": 68.8584757332856,
            "extra_trees": False,
            "feature_fraction": 0.6189215809382,
            "lambda_l1": 0.1641757352921,
            "lambda_l2": 0.6937755557881,
            "learning_rate": 0.0154031028561,
            "max_cat_to_onehot": 17,
            "min_data_in_leaf": 1,
            "min_data_per_group": 30,
            "num_leaves": 68,
        },
    ],
    "REALTABPFN-V2": [
        {
            "ag_args": {"name_suffix": "_r13", "priority": -1},
            "ag_args_ensemble": {"model_random_seed": 104, "vary_seed_across_folds": True},
            "balance_probabilities": False,
            "inference_config/OUTLIER_REMOVAL_STD": 6,
            "inference_config/POLYNOMIAL_FEATURES": "no",
            "inference_config/REGRESSION_Y_PREPROCESS_TRANSFORMS": [None, "safepower"],
            "preprocessing/append_original": False,
            "preprocessing/categoricals": "numeric",
            "preprocessing/global": None,
            "preprocessing/scaling": ["squashing_scaler_default", "quantile_uni_coarse"],
            "softmax_temperature": 1.0,
            "zip_model_path": ["tabpfn-v2-classifier-finetuned-zk73skhh.ckpt", "tabpfn-v2-regressor-v2_default.ckpt"],
        },
        {
            "ag_args": {"name_suffix": "_r106", "priority": -2},
            "ag_args_ensemble": {"model_random_seed": 848, "vary_seed_across_folds": True},
            "balance_probabilities": False,
            "inference_config/OUTLIER_REMOVAL_STD": 6,
            "inference_config/POLYNOMIAL_FEATURES": "no",
            "inference_config/REGRESSION_Y_PREPROCESS_TRANSFORMS": [None],
            "preprocessing/append_original": True,
            "preprocessing/categoricals": "numeric",
            "preprocessing/global": "svd_quarter_components",
            "preprocessing/scaling": ["quantile_uni_coarse"],
            "softmax_temperature": 0.8,
            "zip_model_path": ["tabpfn-v2-classifier-finetuned-zk73skhh.ckpt", "tabpfn-v2-regressor-v2_default.ckpt"],
        },
        {
            "ag_args": {"name_suffix": "_r11", "priority": -6},
            "ag_args_ensemble": {"model_random_seed": 88, "vary_seed_across_folds": True},
            "balance_probabilities": True,
            "inference_config/OUTLIER_REMOVAL_STD": 6,
            "inference_config/POLYNOMIAL_FEATURES": 25,
            "inference_config/REGRESSION_Y_PREPROCESS_TRANSFORMS": [None],
            "preprocessing/append_original": True,
            "preprocessing/categoricals": "onehot",
            "preprocessing/global": "svd_quarter_components",
            "preprocessing/scaling": ["safepower", "quantile_uni"],
            "softmax_temperature": 0.7,
            "zip_model_path": ["tabpfn-v2-classifier-finetuned-zk73skhh.ckpt", "tabpfn-v2-regressor-v2_default.ckpt"],
        },
        {
            "ag_args": {"name_suffix": "_c1", "priority": -8},
            "ag_args_ensemble": {"model_random_seed": 0, "vary_seed_across_folds": True},
            "zip_model_path": ["tabpfn-v2-classifier-finetuned-zk73skhh.ckpt", "tabpfn-v2-regressor-v2_default.ckpt"],
        },
        {
            "ag_args": {"name_suffix": "_r196", "priority": -10},
            "ag_args_ensemble": {"model_random_seed": 1568, "vary_seed_across_folds": True},
            "balance_probabilities": False,
            "inference_config/OUTLIER_REMOVAL_STD": 12,
            "inference_config/POLYNOMIAL_FEATURES": "no",
            "inference_config/REGRESSION_Y_PREPROCESS_TRANSFORMS": ["kdi_alpha_1.0"],
            "preprocessing/append_original": False,
            "preprocessing/categoricals": "numeric",
            "preprocessing/global": None,
            "preprocessing/scaling": ["squashing_scaler_default"],
            "softmax_temperature": 1.25,
            "zip_model_path": ["tabpfn-v2-classifier-finetuned-zk73skhh.ckpt", "tabpfn-v2-regressor-v2_default.ckpt"],
        },
    ],
}
